/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999-2003 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : alleg/gui.c
 *  Version    : 1.5.5
 *  Cr par   : Eric Botcazou 1999
 *  Modifi par: Eric Botcazou 04/11/2003
 *
 *  Panneau de contrle de l'mulateur.
 */


#ifndef SCAN_DEPEND
   #include <stdio.h>
   #include <string.h>
   #include <allegro.h>
#endif

#include "alleg/main.h"
#include "alleg/sound.h"
#include "to7.h"


#define FG_COLOR       3
#define BG_COLOR       4
#define BG_ENTRY_COLOR 5


/* Facteur de correction pour la taille des boutons. */
#define BUTTON_FIX 6

/* Titre commun des botes de dialogue. */
static char title[35];



/* Message affich dans la bote de dialogue. */
static char mesg[35];

/* Bote de dialogue. */
static DIALOG mesgdial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc, 10,   10,    0,   50,     0,     0,   0,    0,       0,    0,    NULL },
{ d_ctext_proc,      10,   20,    0,    0,     0,     0,   0,    0,       0,    0,    mesg },
{ d_button_proc,     30,   40,   32,   16,     0,     0, 'o',    D_EXIT,  0,    0,    "&OK" },
{ d_yield_proc,      10,   10,    0,    0,     0,     0,   0,    0,       0,    0,    NULL },
{ NULL,               0,    0,    0,    0,     0,     0,   0,    0,       0,    0,    NULL }
};

#define MESGDIAL_SHADOW   0
#define MESGDIAL_MESG     1
#define MESGDIAL_OK       2


/* PopupMessage:
 *  Affiche une bote de dialogue contenant le message et un bouton OK.
 */
static void PopupMessage(const char message[])
{
    mesgdial[MESGDIAL_SHADOW].w=strlen(message)*8+16;
    mesgdial[MESGDIAL_MESG].x=mesgdial[MESGDIAL_SHADOW].x+mesgdial[MESGDIAL_SHADOW].w/2;
    mesgdial[MESGDIAL_OK].x=mesgdial[MESGDIAL_MESG].x-(mesgdial[MESGDIAL_OK].w+BUTTON_FIX)/2;

    strcpy(mesg, message);

    centre_dialog(mesgdial);

    popup_dialog(mesgdial, MESGDIAL_OK);
}



/* Question pose dans la bote de dialogue. */
static char quest[35];

/* Bote de dialogue. */
static DIALOG questdial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc, 10,   10,    0,   50,     0,     0,   0,    0,       0,    0,    NULL },
{ d_ctext_proc,      10,   20,    0,    0,     0,     0,   0,    0,       0,    0,    quest },
{ d_button_proc,     30,   40,   32,   16,     0,     0, 'o',    D_EXIT,  0,    0,    "&Oui" },
{ d_button_proc,    130,   40,   32,   16,     0,     0, 'n',    D_EXIT,  0,    0,    "&Non" },
{ d_yield_proc,      10,   10,    0,    0,     0,     0,   0,    0,       0,    0,    NULL },
{ NULL,               0,    0,    0,    0,     0,     0,   0,    0,       0,    0,    NULL }
};

#define QUESTDIAL_SHADOW  0
#define QUESTDIAL_QUEST   1
#define QUESTDIAL_YES     2
#define QUESTDIAL_NO      3

#define FOCUS_YES  QUESTDIAL_YES
#define FOCUS_NO   QUESTDIAL_NO


/* PopupQuestion:
 *  Affiche une bote de dialogue contenant une question et deux boutons Oui/Non.
 */
static int PopupQuestion(const char question[], int focus)
{
    int esp;

    questdial[QUESTDIAL_SHADOW].w=strlen(question)*8+16;
    questdial[QUESTDIAL_QUEST].x=questdial[QUESTDIAL_SHADOW].x+questdial[QUESTDIAL_SHADOW].w/2;

    esp=(questdial[QUESTDIAL_SHADOW].w-2*(questdial[QUESTDIAL_YES].w+BUTTON_FIX))/3;
    questdial[QUESTDIAL_YES].x=questdial[QUESTDIAL_SHADOW].x+esp;
    questdial[QUESTDIAL_NO].x=questdial[QUESTDIAL_YES].x+(questdial[QUESTDIAL_YES].w+BUTTON_FIX)+esp;

    strcpy(quest, question);

    centre_dialog(questdial);

    return (popup_dialog(questdial, focus) == QUESTDIAL_YES ? TRUE : FALSE);
}



/* Noms des fichiers utiliss comme disquettes. */
#define LABEL_LENGTH     20
static char disk_label[4][LABEL_LENGTH+1];

/* Masque de bits dcrivant les lecteurs supportant la lecture directe. */
static int direct_disk;

/* Bote de dialogue. */
static DIALOG diskdial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc, 20,   10,  280,  180,     0,     0,   0,    0,       0,    0,    NULL },
{ d_ctext_proc,     160,   20,    0,    0,     0,     0,   0,    0,       0,    0,    title },
{ d_ctext_proc,     160,   30,    0,    0,     0,     0,   0,    0,       0,    0,    "Lecteurs de disquettes" },
  /* disk 0 */
{ d_text_proc,       30,   54,    0,    0,     0,     0,   0,    0,       0,    0,    "0:" },
{ d_textbox_proc,    47,   50,  167,   16,     0,     0,   0,    0,       0,    0,    disk_label[0] },
{ d_button_proc,    220,   50,   30,   16,     0,     0, '0',    D_EXIT,  0,    0,    "..." },
{ d_check_proc,     260,   50,   15,   15,     0,     0,   0,    D_EXIT,  0,    0,    "" },
  /* disk 1 */
{ d_text_proc,       30,   78,    0,    0,     0,     0,   0,    0,       0,    0,    "1:" },
{ d_textbox_proc,    47,   74,  167,   16,     0,     0,   0,    0,       0,    0,    disk_label[1] },
{ d_button_proc,    220,   74,   30,   16,     0,     0, '1',    D_EXIT,  0,    0,    "..." },
{ d_check_proc,     260,   74,   15,   15,     0,     0,   0,    D_EXIT,  0,    0,    "" },
  /* disk 2 */
{ d_text_proc,       30,  102,    0,    0,     0,     0,   0,    0,       0,    0,    "2:" },
{ d_textbox_proc,    47,   98,  167,   16,     0,     0,   0,    0,       0,    0,    disk_label[2] },
{ d_button_proc,    220,   98,   30,   16,     0,     0, '2',    D_EXIT,  0,    0,    "..." },
{ d_check_proc,     260,   98,   15,   15,     0,     0,   0,    D_EXIT,  0,    0,    "" },
  /* disk 3 */
{ d_text_proc,       30,  126,    0,    0,     0,     0,   0,    0,       0,    0,    "3:" },
{ d_textbox_proc,    47,  122,  167,   16,     0,     0,   0,    0,       0,    0,    disk_label[3] },
{ d_button_proc,    220,  122,   30,   16,     0,     0, '3',    D_EXIT,  0,    0,    "..." },
{ d_check_proc,     260,  122,   15,   15,     0,     0,   0,    D_EXIT,  0,    0,    "" },
  /* direct disk */
{ d_button_proc,     30,  146,  250,   16,     0,     0, 'd',    D_EXIT,  0,    0,    "Accs &direct" },
{ d_text_proc,      255,   40,    0,    0,     0,     0,   0,    0,       0,    0,    "prot." },
{ d_button_proc,     30,  170,  126,   16,     0,     0, 'o',    D_EXIT,  0,    0,    "&OK" },
{ d_yield_proc,      20,   10,    0,    0,     0,     0,   0,    0,       0,    0,    NULL },
{ NULL,               0,    0,    0,    0,     0,     0,   0,    0,       0,    0,    NULL }
};

#define DISKDIAL_LABEL0    4
#define DISKDIAL_BUTTON0   5
#define DISKDIAL_CHECK0    6
#define DISKDIAL_LABEL1    8
#define DISKDIAL_BUTTON1   9
#define DISKDIAL_CHECK1    10
#define DISKDIAL_LABEL2    12
#define DISKDIAL_BUTTON2   13
#define DISKDIAL_CHECK2    14
#define DISKDIAL_LABEL3    16
#define DISKDIAL_BUTTON3   17
#define DISKDIAL_CHECK3    18
#define DISKDIAL_DIRECT    19
#define DISKDIAL_OK        21


/* MenuDisk:
 *  Affiche le menu de gestion des lecteurs de disquettes.
 */
static void MenuDisk(void)
{
    static int first=1;
    static char filename[FILENAME_LENGTH];
    int drive, ret, ret2;
    
    if (first)
    {
        /* La premire fois on tente d'ouvrir le rpertoire par dfaut. */
        if (file_exists("./disks", FA_DIREC, NULL))
	    strcpy(filename,"./disks/");
	
        centre_dialog(diskdial);

        for (drive=0; drive<4; drive++)
        {
            strncpy(disk_label[drive], get_filename(to7_GetDiskFilename(drive)), LABEL_LENGTH);
            if (disk_label[drive][0] == '\0')
                strncpy(disk_label[drive], "aucune disquette", LABEL_LENGTH);
        }

	first=0;
    }

    clear_keybuf();

    while (TRUE)
    {
        ret=popup_dialog(diskdial, DISKDIAL_OK);
        
        switch (ret)
        {
            case DISKDIAL_BUTTON0:
            case DISKDIAL_BUTTON1:
            case DISKDIAL_BUTTON2:
            case DISKDIAL_BUTTON3:
                drive=(ret-5)/4;
            
                if (file_select_ex("Choisissez votre disquette:", filename, "sap",
                                   FILENAME_LENGTH, OLD_FILESEL_WIDTH, OLD_FILESEL_HEIGHT))
                {
                    ret2=to7_LoadDisk(drive, filename);

                    if (ret2 == TO7_ERROR)
                        PopupMessage(to7_error_msg);
                    else
                    {
                        strncpy(disk_label[drive], get_filename(filename), LABEL_LENGTH);

                        if ((ret2==TO7_READ_ONLY) && !(diskdial[ret+1].d2))
                        {
                            PopupMessage("Attention: criture impossible.");
                            diskdial[ret+1].flags|=D_SELECTED;
                            diskdial[ret+1].d2=1;
                        }
                    }
                }
                break;

            case DISKDIAL_CHECK0:
            case DISKDIAL_CHECK1:
            case DISKDIAL_CHECK2:
            case DISKDIAL_CHECK3:
                drive=(ret-6)/4;
            
                if (diskdial[ret].d2)
                {
                    if (to7_SetDiskMode(drive, TO7_READ_WRITE)==TO7_READ_ONLY)
                    {
                        PopupMessage("Ecriture impossible sur ce support.");
                        diskdial[ret].flags|=D_SELECTED;
                        diskdial[ret].d2=1;
                    }
                    else
                    {
                        diskdial[ret].flags&=~D_SELECTED;
                        diskdial[ret].d2=0;
                    }
                }
                else
                {
                    to7_SetDiskMode(drive, TO7_READ_ONLY);
                    diskdial[ret].flags|=D_SELECTED;
                    diskdial[ret].d2=1;
                }
                break;

            case DISKDIAL_DIRECT:
                for (drive=0; drive<4; drive++)
                {
                    if (direct_disk & (1<<drive))
                    {
                        strncpy(disk_label[drive], "accs direct", LABEL_LENGTH);

                        if (to7_DirectSetDrive(drive) == TO7_READ_ONLY)
                        {
                            diskdial[DISKDIAL_CHECK0+4*drive].flags|=D_SELECTED;
                            diskdial[DISKDIAL_CHECK0+4*drive].d2=1;
                        }
                        else
                        {
                            diskdial[DISKDIAL_CHECK0+4*drive].flags&=~D_SELECTED;
                            diskdial[DISKDIAL_CHECK0+4*drive].d2=0;
                        }
                    }
                }
                break;

            case -1:  /* ESC */
            case DISKDIAL_OK:
                return;
        }
    }
}



/* Nom du fichier utilis comme cassette. */
static char k7_label[LABEL_LENGTH+1];

/* Affichage du compteur. */
#define COUNTER_MAX  999
static char k7_counter_str[4];

/* Horloge de la rptition automatique. */
static int k7_tick;

/* Rfrences avant. */
static int updown_edit_proc(int msg, DIALOG *d, int c);
static int up_button_proc(int msg, DIALOG *d, int c);
static int down_button_proc(int msg, DIALOG *d, int c);

/* Bote de dialogue. */
static DIALOG k7dial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc,  20,  10,  280,  180,     0,     0,    0,    0,       0,    0,    NULL },
{ d_ctext_proc,      160,  20,    0,    0,     0,     0,    0,    0,       0,    0,    title },
{ d_ctext_proc,      160,  30,    0,    0,     0,     0,    0,    0,       0,    0,    "Lecteur de cassettes" },
{ d_text_proc,        30,  54,    0,    0,     0,     0,    0,    0,       0,    0,    "k7" },
{ d_textbox_proc,     47,  50,  167,   16,     0,     0,    0,    0,       0,    0,    k7_label },
{ d_button_proc,     220,  50,   30,   16,     0,     0,  'k',    D_EXIT,  0,    0,    "..." },
{ d_check_proc,      260,  50,   15,   15,     0,     0,    0,    D_EXIT|D_SELECTED,0,1,"" },
{ d_text_proc,       255,  40,    0,    0,     0,     0,    0,    0,       0,    0,    "prot." },
{ d_text_proc,        30,  79,    0,    0,     0,     0,    0,    0,       0,    0,    "Compteur:" },
{ d_box_proc,        106,  77,   36,   12,     0,     0,    0,    0,       0,    0,    NULL },
{ updown_edit_proc,  108,  79,   32,   10,     0,     0,    0,    0,       3,    3,    k7_counter_str },
{ up_button_proc,    145,  73,   19,   10,     0,     0,    0,    D_EXIT,  0,    0,    "+" },
{ down_button_proc,  145,  84,   19,   10,     0,     0,    0,    0,       0,    0,    "-" },
{ d_button_proc,     174,  75,  106,   16,     0,     0,  'r',    D_EXIT,  0,    0,    "&Rembobiner" },
{ d_button_proc,      30, 170,  126,   16,     0,     0,  'o',    D_EXIT,  0,    0,    "&OK" },
{ d_yield_proc,       20,  10,    0,    0,     0,     0,    0,    0,       0,    0,    NULL },
{ NULL,                0,   0,    0,    0,     0,     0,    0,    0,       0,    0,    NULL }
};

#define K7DIAL_LABEL    4
#define K7DIAL_BUTTON   5
#define K7DIAL_CHECK    6
#define K7DIAL_BOXEDIT  9
#define K7DIAL_EDIT     10
#define K7DIAL_REWIND   13
#define K7DIAL_OK       14


/* updown_edit_proc:
 *  Procdure du champ d'entre du compteur.
 */
static int updown_edit_proc(int msg, DIALOG *d, int c)
{
    char *endp;
    int ret, counter;

    /* Seuls les chiffres nous intressent. */
    if ((msg == MSG_UCHAR) && !uisdigit(c))
        return D_O_K;

    ret = d_edit_proc(msg, d, c);

    /* d_edit_proc peut ne rien faire  la rception de MSG_CHAR et attendre
       le MSG_UCHAR suivant. Par consquent D_USED_CHAR est le bon signal. */
    if (ret == D_USED_CHAR)
    {
        counter = ustrtol(k7_counter_str, &endp, 10);
        if (endp == k7_counter_str)
            counter = 0;
        else
            counter = MID(0, counter, COUNTER_MAX);

        to7_SetK7Counter(counter);
    }

    return ret;
}


/* update_counter:
 *  Met  jour le compteur et son affichage.
 */
static void update_counter(int *counter)
{
    *counter = MID(0, *counter, COUNTER_MAX);
    to7_SetK7Counter(*counter);

    k7dial[K7DIAL_EDIT].d2 = uszprintf(k7_counter_str, sizeof(k7_counter_str), "%d", *counter);
    scare_mouse();
    object_message(k7dial+K7DIAL_EDIT, MSG_DRAW, 0);
    unscare_mouse();
}


/* updown_button_proc:
 *  Corps de procdure des boutons UP et DOWN.
 */
static int updown_button_proc(int msg, DIALOG *d, int c, int down)
{
    int previous_tick, step, counter;

    if (msg == MSG_CLICK)
    {
        /* Enfonce le bouton. */
        d->flags |= D_SELECTED;
        scare_mouse();
        object_message(d, MSG_DRAW, 0);
        unscare_mouse();

        counter = to7_GetK7Counter();

        if (down)
           counter--;
        else
           counter++;

        update_counter(&counter);

        /* Gestion de la rptition automatique. */
        k7_tick = 0;
        previous_tick = 1;
        while (gui_mouse_b())
        {
            if (k7_tick > previous_tick)
            {
                if (k7_tick < 5)
                    step = 1;
                else if (k7_tick < 15)
                    step = 10;
                else
                    step = 50;
 
                if (down)
                   counter -= step;
                else
                   counter += step;

                update_counter(&counter);

                previous_tick = k7_tick;
            }

   	    /* Poursuit l'animation des autres objets. */
	    broadcast_dialog_message(MSG_IDLE, 0);
        }

        /* Relche le bouton. */
        d->flags &= ~D_SELECTED;
        scare_mouse();
        object_message(d, MSG_DRAW, 0);
        unscare_mouse();

        return D_O_K;
    }
  
    return d_button_proc(msg, d, c);
}


/* up_button_proc:
 *  Procdure du boutons UP.
 */
static int up_button_proc(int msg, DIALOG *d, int c)
{
    return updown_button_proc(msg, d, c, FALSE);
}


/* down_button_proc:
 *  Procdure des boutons DOWN.
 */
static int down_button_proc(int msg, DIALOG *d, int c)
{
    return updown_button_proc(msg, d, c, TRUE);
}


/* k7_ticker:
 *  Mcanisme de l'horloge de la rptition automatique.
 */
static void k7_ticker(void)
{
    k7_tick++;
}


/* MenuK7:
 *  Affiche le menu de gestion du lecteur de cassettes.
 */
static void MenuK7(void)
{
    static int first=1;
    static char filename[FILENAME_LENGTH];
    int ret;

    if (first)
    {
        /* La premire fois on tente d'ouvrir le rpertoire par dfaut. */
    	if (file_exists("./k7", FA_DIREC, NULL))
	    strcpy(filename,"./k7/");

        centre_dialog(k7dial);

        strncpy(k7_label, get_filename(to7_GetK7Filename()), LABEL_LENGTH);
        if (k7_label[0] == '\0')
            strncpy(k7_label, "aucune cassette", LABEL_LENGTH);

	first=0;
    }

    clear_keybuf();
    install_int_ex(k7_ticker, BPS_TO_TIMER(2));

    while (TRUE)
    {
        k7dial[K7DIAL_EDIT].d2 = uszprintf(k7_counter_str, sizeof(k7_counter_str), "%d", to7_GetK7Counter());

        ret=popup_dialog(k7dial, K7DIAL_OK);
        
        switch (ret)
        {
            case K7DIAL_BUTTON:
                if (file_select_ex("Choisissez votre cassette:", filename, "k7",
                                   FILENAME_LENGTH, OLD_FILESEL_WIDTH, OLD_FILESEL_HEIGHT))
                {
                    ret=to7_LoadK7(filename);

                    if (ret == TO7_ERROR)
                        PopupMessage(to7_error_msg);
                    else
                    {
                        strncpy(k7_label, get_filename(filename), LABEL_LENGTH);

                        if ((ret==TO7_READ_ONLY) && !(k7dial[6].d2))
                        {
                            PopupMessage("Attention: criture impossible.");
                            k7dial[K7DIAL_CHECK].flags|=D_SELECTED;
                            k7dial[K7DIAL_CHECK].d2=1;
                        }
                    }
                }
                break;

            case K7DIAL_CHECK:
                if (k7dial[K7DIAL_CHECK].d2)
                {
                    if (to7_SetK7Mode(TO7_READ_WRITE)==TO7_READ_ONLY)
                    {
                        PopupMessage("Ecriture impossible sur ce support.");
                        k7dial[K7DIAL_CHECK].flags|=D_SELECTED;
                        k7dial[K7DIAL_CHECK].d2=1;
                    }
                    else
                    {
                        k7dial[K7DIAL_CHECK].flags&=~D_SELECTED;
                        k7dial[K7DIAL_CHECK].d2=0;
                    }
                }
                else
                {
                    to7_SetK7Mode(TO7_READ_ONLY);
                    k7dial[K7DIAL_CHECK].flags|=D_SELECTED;
                    k7dial[K7DIAL_CHECK].d2=1;
                }
                break;

            case K7DIAL_REWIND:
                to7_SetK7Counter(0);
                break;

            case -1:  /* ESC */
            case K7DIAL_OK:
                remove_int(k7_ticker);
                return;
        }
    }
}



/* Nom du fichier utilis comme cartouche. */
static char m7_label[TO7_MEMO7_LABEL_LENGTH+1];

/* Bote de dialogue. */
static DIALOG m7dial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg)  (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc,  20,  10,  280,  180,     0,     0,    0,    0,       0,    0,    NULL },
{ d_ctext_proc,      160,  20,    0,    0,     0,     0,    0,    0,       0,    0,    title },
{ d_ctext_proc,      160,  30,    0,    0,     0,     0,    0,    0,       0,    0,    "Lecteur de cartouches" },
{ d_text_proc,        30,  54,    0,    0,     0,     0,    0,    0,       0,    0,    "m7" },
{ d_textbox_proc,     47,  50,  208,   16,     0,     0,    0,    0,       0,    0,    m7_label },
{ d_button_proc,     260,  50,   30,   16,     0,     0,  'm',    D_EXIT,  0,    0,    "..." },
{ d_button_proc,      30, 170,  126,   16,     0,     0,  'o',    D_EXIT,  0,    0,    "&OK" },
{ d_yield_proc,       20,  10,    0,    0,     0,     0,    0,    0,       0,    0,    NULL },
{ NULL,                0,   0,    0,    0,     0,     0,    0,    0,       0,    0,    NULL }
};

#define M7DIAL_LABEL   4
#define M7DIAL_BUTTON  5
#define M7DIAL_OK      6


/* MenuMemo7:
 *  Affiche le menu de gestion du lecteur de cartouches.
 */
static void MenuMemo7(void)
{
    static int first=1;
    static char filename[FILENAME_LENGTH];
    int ret;

    if (first)
    {
        /* La premire fois on tente d'ouvrir le rpertoire par dfaut. */
	if (file_exists("./memo7", FA_DIREC, NULL))
	    strcpy(filename,"./memo7/");
	
        centre_dialog(m7dial);

        strncpy(m7_label, to7_GetMemo7Label(), TO7_MEMO7_LABEL_LENGTH);
        if (m7_label[0] == '\0')
            strncpy(m7_label, "aucune cartouche", TO7_MEMO7_LABEL_LENGTH);

	first=0;
    }

    clear_keybuf();

    while (TRUE)
    {
        ret=popup_dialog(m7dial, M7DIAL_OK);

        switch (ret)
        {
            case M7DIAL_BUTTON:
                if (file_select_ex("Choisissez votre cartouche:", filename, "m7",
                                   FILENAME_LENGTH, OLD_FILESEL_WIDTH, OLD_FILESEL_HEIGHT))
                {
                    if (to7_LoadMemo7(filename) == TO7_ERROR)                    
                        PopupMessage(to7_error_msg);
                    else
                    {
                        strncpy(m7_label, to7_GetMemo7Label(), TO7_MEMO7_LABEL_LENGTH);
                        thom.command=COLD_RESET;
                    }
                }
                break;

            case -1:  /* ESC */
            case M7DIAL_OK:
                return;
        }
    }
}



/* Bote de dialogue. */
static DIALOG commdial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)    (fg)   (bg)   (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc,  20,  10,   280,  180,     0,     0,      0,   0,       0,    0,    NULL },
{ d_ctext_proc,      160,  20,     0,    0,     0,     0,      0,   0,       0,    0,    title },
{ d_ctext_proc,      160,  30,     0,    0,     0,     0,      0,   0,       0,    0,    "Commandes et Rglages" },
{ d_button_proc,      30,  50,   260,   16,     0,     0,    'r',  D_EXIT,   0,    0,    "&Rinitialiser le TO7-70" },
{ d_button_proc,      30,  74,   260,   16,     0,     0,    'f',  D_EXIT,   0,    0,    "&Redmarrer  &froid le TO7-70" },
{ d_text_proc,        30, 100,     0,    0,     0,     0,      0,   0,       0,    0,    "Vitesse:" },
{ d_radio_proc,      105, 100,   126,    8,     0,     0,    'e',   0,       1,    0,    "&exacte" },
{ d_radio_proc,      105, 114,   126,    8,     0,     0,    'm',   0,       1,    0,    "&maximale" },
{ d_text_proc,        30, 138,     0,    0,     0,     0,      0,   0,       0,    0,    "Volume &sonore:" },
{ d_slider_proc,     160, 135,   100,   15,     0,     0,    's',   0,     254,    0,    NULL},
{ d_text_proc,       150, 152,     0,    0,     0,     0,      0,   0,       0,    0,    "bas         haut" },
{ d_button_proc,      30, 170,   126,   16,     0,     0,    'o',   D_EXIT,  0,    0,    "&OK" },
{ d_yield_proc,       20,  10,     0,    0,     0,     0,      0,   0,       0,    0,    NULL },
{ NULL,                0,   0,     0,    0,     0,     0,      0,   0,       0,    0,    NULL }
};

#define COMMDIAL_INIT        3
#define COMMDIAL_COLDINIT    4
#define COMMDIAL_EXACTSPEED  6
#define COMMDIAL_MAXSPEED    7
#define COMMDIAL_VOLUME      8
#define COMMDIAL_SLIDER      9
#define COMMDIAL_SPAN        10
#define COMMDIAL_OK          11


/* MenuComm:
 *  Affiche le menu des commandes et rglages.
 */
static void MenuComm(void)
{
    if (thom.exact_speed)
    {
        commdial[COMMDIAL_EXACTSPEED].flags=D_SELECTED;
        commdial[COMMDIAL_MAXSPEED].flags=0;
    }
    else
    {
        commdial[COMMDIAL_EXACTSPEED].flags=0;
        commdial[COMMDIAL_MAXSPEED].flags=D_SELECTED;
    }

    if (thom.sound_enabled && thom.exact_speed)
        commdial[COMMDIAL_SLIDER].d2=GetVolume()-1;

    clear_keybuf();
    centre_dialog(commdial);

    switch (popup_dialog(commdial, COMMDIAL_OK))
    {
        case COMMDIAL_INIT:
            thom.command=RESET;
            break;

        case COMMDIAL_COLDINIT:
            thom.command=COLD_RESET;
            break;
    }

    if (thom.sound_enabled && thom.exact_speed)
        SetVolume(commdial[COMMDIAL_SLIDER].d2+1);

    thom.exact_speed=(commdial[COMMDIAL_EXACTSPEED].flags&D_SELECTED ? TRUE : FALSE);
}



/* Bote de dialogue. */
static DIALOG controldial[]={
/* (dialog proc)     (x)   (y)   (w)   (h)   (fg)   (bg) (key) (flags)  (d1)  (d2)  (dp) */
{ d_shadow_box_proc,  20,  10,  280,  180,     0,     0,   0,    0,       0,    0,    NULL },
{ d_ctext_proc,      160,  20,    0,    0,     0,     0,   0,    0,       0,    0,    title },
{ d_ctext_proc,      160,  30,    0,    0,     0,     0,   0,    0,       0,    0,    "Sylvain Huet - Eric Botcazou" },
{ d_button_proc,     30,   60,  260,   16,     0,     0,   'c',  D_EXIT,  0,    0,    "&Commandes et Rglages..." },
{ d_button_proc,     30,   84,  260,   16,     0,     0,   't',  D_EXIT,  0,    0,    "Lecteur de car&touches..." },
{ d_button_proc,     30,  108,  260,   16,     0,     0,   's',  D_EXIT,  0,    0,    "Lecteur de ca&ssettes..." },
{ d_button_proc,     30,  132,  260,   16,     0,     0,   'd',  D_EXIT,  0,    0,    "Lecteurs de &disquettes..." },
{ d_button_proc,     30,  170,  126,   16,     0,     0,   'o',  D_EXIT,  0,    0,    "&OK" },
{ d_button_proc,    164,  170,  126,   16,     0,     0,   'q',  D_EXIT,  0,    0,    "&Quitter" },
{ d_yield_proc,      20,   10,    0,    0,     0,     0,   0,    0,       0,    0,    NULL },
{ NULL,               0,    0,    0,    0,     0,     0,   0,    0,       0,    0,    NULL }
};

#define CONTROLDIAL_COMMAND   3
#define CONTROLDIAL_CART      4
#define CONTROLDIAL_CASS      5
#define CONTROLDIAL_DISK      6
#define CONTROLDIAL_OK        7
#define CONTROLDIAL_QUIT      8


/* ControlPanel:
 *  Affiche le panneau de contrle.
 */
void ControlPanel(void)
{
    clear_keybuf();
    centre_dialog(controldial);

    while (TRUE)
        switch (popup_dialog(controldial, CONTROLDIAL_OK))
        {
            case CONTROLDIAL_COMMAND:
                MenuComm();
                break;

            case CONTROLDIAL_CART:
                MenuMemo7();
                break;

            case CONTROLDIAL_CASS:
                MenuK7();
                break;

            case CONTROLDIAL_DISK:
                MenuDisk();
                break;

            case -1:  /* ESC */
            case CONTROLDIAL_OK:
                return;

            case CONTROLDIAL_QUIT:
                if (PopupQuestion("Voulez-vous vraiment quitter ?", FOCUS_NO))
                {
                    thom.command=QUIT;
                    return;
                }
        }
}



/* InitGUI:
 *  Initialise le module interface utilisateur.
 */
void InitGUI(const char version_name[], int disk_support, int direct_disk_support)
{
    strcpy(title, version_name);

    if (!thom.sound_enabled)
        commdial[COMMDIAL_VOLUME].flags=commdial[COMMDIAL_SPAN].flags=commdial[COMMDIAL_SLIDER].flags=D_DISABLED;

    if (!disk_support)
       controldial[CONTROLDIAL_DISK].flags=D_HIDDEN;

    if (!direct_disk_support)
        diskdial[DISKDIAL_DIRECT].flags=D_HIDDEN;

    direct_disk = direct_disk_support;

    if (strlen(to7_GetMemo7Label()))
        strncpy(m7_label, to7_GetMemo7Label(), TO7_MEMO7_LABEL_LENGTH);

    set_dialog_color( mesgdial, FG_COLOR, BG_COLOR);
    set_dialog_color(questdial, FG_COLOR, BG_COLOR);

    set_dialog_color(diskdial, FG_COLOR, BG_COLOR);
    diskdial[DISKDIAL_LABEL0].bg = BG_ENTRY_COLOR;
    diskdial[DISKDIAL_LABEL1].bg = BG_ENTRY_COLOR;
    diskdial[DISKDIAL_LABEL2].bg = BG_ENTRY_COLOR;
    diskdial[DISKDIAL_LABEL3].bg = BG_ENTRY_COLOR;

    set_dialog_color(k7dial, FG_COLOR, BG_COLOR);
    k7dial[K7DIAL_LABEL].bg = BG_ENTRY_COLOR;
    k7dial[K7DIAL_BOXEDIT].bg = BG_ENTRY_COLOR;
    k7dial[K7DIAL_EDIT].bg = BG_ENTRY_COLOR;

    set_dialog_color(m7dial, FG_COLOR, BG_COLOR);
    m7dial[M7DIAL_LABEL].bg = BG_ENTRY_COLOR;
    
    set_dialog_color(commdial, FG_COLOR, BG_COLOR);
    set_dialog_color(controldial, FG_COLOR, BG_COLOR);

    /* Pour tous les objets. */
    gui_fg_color = FG_COLOR;
    gui_bg_color = BG_COLOR;
    gui_mg_color = BG_COLOR;
}

