/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999-2002 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : dos/debug.c
 *  Version    : 1.5.5
 *  Cr par   : Eric Botcazou fvrier 1999
 *  Modifi par: Eric Botcazou 14/07/2000
 *
 *  Dbogueur du TO7-70.
 */


#ifndef SCAN_DEPEND
   #include <conio.h>
   #include <stdio.h>
#endif

#include "dos/memmng.h"
#include "alleg/graphic.h"
#include "mc68xx/dasm6809.h"
#include "mc68xx/mc6809.h"
#include "to7dbg.h"


#ifdef DEBUG
   #define MENU_NLINES 7
#else
   #define MENU_NLINES 6
#endif

#define MENU_POS_X  4
#define MENU_POS_Y 43

static char menu_line[MENU_NLINES][50]={ "Commandes:",
                                         " s: xcuter l'instruction suivante",
                                         " d: xcuter jusqu' dpasser le point actuel",
#ifdef DEBUG
                                         " t: active/dsactive le mode trace",
#endif
                                         " a: afficher l'cran du TO7-70",
                                         " m: lancer le gestionnaire de mmoire du TO7-70",
                                         " q: quitter le dbogueur"};

#define DASM_NLINES 40

static struct MC6809_REGS regs;

#ifdef DEBUG
#define FTRACE_NLINES 4
#define FTRACE_POS_X 55
#define FTRACE_POS_Y 44

static char ftrace_line[FTRACE_NLINES][17]={ "ͻ",
                                             "  Mode Trace  ",
                                             "              ",
                                             "ͼ"};
static int ftrace_mode=0;
static char ftrace_filename[]="ftrace.txt";
#endif



/* DisplayRegs:
 *  Affiche les registres du 6809.
 */
static void DisplayRegs(void)
{
    textbackground(BLUE);
    textcolor(LIGHTGRAY);

    gotoxy(1,1);
    cprintf("        CPU MC6809E           ");

    gotoxy(1,11);
    cprintf("      PIA 6846 systme        ");

    gotoxy(1,16);
    cprintf("      PIA 6821 systme        ");

    gotoxy(1,20);
    cprintf("   PIA 6821 musique et jeux   ");

    gotoxy(1,24);
    cprintf("       Logic Gate Array       ");

    gotoxy(1,28);
    cprintf("   Statut des pages mmoire   ");

    textbackground(BLACK);
    textcolor(WHITE);

    gotoxy(2,2);
    cprintf("PC: %04X        DP: %02X",regs.pc,regs.dp);
    gotoxy(2,3);
    cprintf(" X: %04X         Y: %04X",regs.xr,regs.yr);
    gotoxy(2,4);
    cprintf(" U: %04X         S: %04X",regs.ur,regs.sr);
    gotoxy(2,5);
    cprintf(" A: %02X           B: %02X",regs.ar,regs.br);
    gotoxy(2,6);
    cprintf("CC: %c%c%c%c%c%c%c%c   IRQ: %d" ,regs.cc&0x80 ? 'E' : '.'
                                             ,regs.cc&0x40 ? 'F' : '.'
                                             ,regs.cc&0x20 ? 'H' : '.'
                                             ,regs.cc&0x10 ? 'I' : '.'
                                             ,regs.cc&0x08 ? 'N' : '.'
                                             ,regs.cc&0x04 ? 'Z' : '.'
                                             ,regs.cc&0x02 ? 'V' : '.'
                                             ,regs.cc&0x01 ? 'C' : '.'
                                             ,mc6809_irq);

    gotoxy(2,8);
    cprintf("[S ] [S+1] [S+2] [S+3] [S+4]");
    gotoxy(2,9);
    cprintf(" %2X   %2X    %2X    %2X    %2X",LOAD_BYTE(regs.sr),LOAD_BYTE(regs.sr+1),
              LOAD_BYTE(regs.sr+2),LOAD_BYTE(regs.sr+3),LOAD_BYTE(regs.sr+4));

    gotoxy(2,12);
    cprintf(" CSR: %02X   CRC: %02X",mc6846.csr,mc6846.crc);
    gotoxy(2,13);
    cprintf("DDRC: %02X   PRC: %02X",mc6846.ddrc,mc6846.prc);
    gotoxy(2,14);
    cprintf(" TCR: %02X  TMSB: %02X  TLSB: %02X",mc6846.tcr,mc6846.tmsb,
      mc6846.tlsb);

    gotoxy(2,17);
    cprintf("CRA: %02X  DDRA: %02X  PDRA: %02X",pia_int.porta.cr,
      pia_int.porta.ddr,mc6821_ReadPort(&pia_int.porta));
    gotoxy(2,18);
    cprintf("CRB: %02X  DDRB: %02X  PDRB: %02X",pia_int.portb.cr,
      pia_int.portb.ddr,mc6821_ReadPort(&pia_int.portb));

    gotoxy(2,21);
    cprintf("CRA: %02X  DDRA: %02X  PDRA: %02X",pia_ext.porta.cr,
      pia_ext.porta.ddr,mc6821_ReadPort(&pia_ext.porta));
    gotoxy(2,22);
    cprintf("CRB: %02X  DDRB: %02X  PDRB: %02X",pia_ext.portb.cr,
      pia_ext.portb.ddr,mc6821_ReadPort(&pia_ext.portb));

    gotoxy(2,25);
    cprintf("LP1: %02X   LP2: %02X", lga.lp1, lga.lp2);
    gotoxy(2,26);
    cprintf("LP3: %02X   LP4: %02X", lga.lp3, lga.lp4);

    gotoxy(2,29);
    cprintf("page de RAM: %d", mempager.data.page);
}



/* Debugger:
 *  Fonction principale du dbogueur.
 */
void Debugger(void)
{
    register int i,j;
             int c=0,pc;
    char fetch_buffer[MC6809_FETCH_BUFFER_SIZE],
         dasm_buffer[MC6809_DASM_BUFFER_SIZE];

    _set_screen_lines(50);
    _setcursortype(_NOCURSOR);

    do
    {
        if ((c=='s') || (c=='S'))
            mc6809_StepExec(1);

        if ((c=='d') || (c=='D'))
        {
            int old_pc=regs.pc;

            while ((regs.pc<=old_pc)||(regs.pc>old_pc+8))
            {
               mc6809_StepExec(1);
               mc6809_GetRegs(&regs);
            }
        }
        
#ifdef DEBUG
        if ((c=='t') || (c=='T'))
        {
            if (ftrace_mode)
            {
                fprintf(mc6809_ftrace, "End of ftrace.\n\n");
                fclose(mc6809_ftrace);
                mc6809_ftrace=NULL;
                ftrace_mode=0;
            }
            else
            {
                if ( (mc6809_ftrace=fopen(ftrace_filename, "a+")) == NULL)
                   fprintf(stderr, "Can't open ftrace file %s\n", ftrace_filename);
                else
                {
                    fprintf(mc6809_ftrace, "Begin of ftrace:\n");
                    ftrace_mode=1;
                }
            }
        }
#endif

        if ((c=='a') || (c=='A'))
        {
            SetGraphicMode(RESTORE);
            getch();
            SetGraphicMode(SHUTDOWN);
            _set_screen_lines(50);
            _setcursortype(_NOCURSOR);
        }

        if ((c=='m') || (c=='M'))
            MemoryManager();

        textbackground(BLACK);
        textcolor(LIGHTCYAN);
        clrscr();

        /* le menu */
        for (i=0; i<MENU_NLINES; i++)
        {
            gotoxy(MENU_POS_X,MENU_POS_Y+i);
            cputs(menu_line[i]);
        }

        mc6809_GetRegs(&regs);
        DisplayRegs();

        pc=regs.pc;

        textbackground(BLUE);
        textcolor(LIGHTGRAY);

        /* le dsassembleur */
        gotoxy(33,1);
        cprintf("              Dsassembleur MC6809E             ");

        textbackground(BLACK);
        textcolor(WHITE);

        for (i=0; i<DASM_NLINES; i++)
        {
            for (j=0; j<5; j++)
                fetch_buffer[j]=LOAD_BYTE(pc+j);

            pc=(pc+MC6809_Dasm(dasm_buffer,fetch_buffer,pc,MC6809_DASM_BINASM_MODE))&0xFFFF;
            gotoxy(33,i+2);
            cputs(dasm_buffer);
        }

#ifdef DEBUG
        for (i=0; i<FTRACE_NLINES; i++)
        {
            gotoxy(FTRACE_POS_X, FTRACE_POS_Y+i);
            cputs(ftrace_line[i]);
        }

        gotoxy(FTRACE_POS_X+4, FTRACE_POS_Y+2);
        if (ftrace_mode)
        {
            textcolor(GREEN);
            cprintf("Activ");
        }
        else
        {
            textcolor(RED);
            cprintf("Dsactiv");
        }
#endif
        
        c=getch();

    } while ((c != 'q') && (c != 'Q'));
}

