/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999-2003 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : linux/graphic.c
 *  Version    : 1.5.5
 *  Cr par   : Eric Botcazou juillet 1999
 *  Modifi par: Eric Botcazou 03/11/2003
 *
 *  Gestion de l'affichage du TO7-70.
 */


#ifndef SCAN_DEPEND
   #include <stdio.h>
   #include <stdlib.h> 
   #include <sys/ipc.h>
   #include <sys/shm.h>
   #include <X11/Xlib.h>
   #include <X11/Xutil.h>
   #include <X11/extensions/XShm.h>
#endif

#include "linux/display.h"
#include "to7.h"


int need_border_refresh;

#define CELL_COLS  40
#define CELL_ROWS  25

static int modified_cell[CELL_COLS+1][CELL_ROWS];
static XVisualInfo visualinfo;
static Colormap colormap;
static GC gc;
static XColor xcolor[TO7_NCOLORS+2];
static XImage *buffer;
static int border_color;



/* Routines d'affichage:
 */
static void (*PutPixel)(XImage *, int, int, unsigned long);

static void PutPixel1(XImage *buffer, int x, int y, unsigned long pixel)
{
    XPutPixel(buffer, x, y, pixel);
}

static void PutPixel2(XImage *buffer, int x, int y, unsigned long pixel)
{
    x<<=1;
    y<<=1;

    XPutPixel(buffer, x,   y,   pixel);
    XPutPixel(buffer, x+1, y,   pixel);
    XPutPixel(buffer, x,   y+1, pixel);
    XPutPixel(buffer, x+1, y+1, pixel);
}



/* RefreshBorder:
 *  Met  jour la couleur du pourtour de l'cran.
 */
void RefreshBorder(void)
{
    XSetWindowBackground(display, main_win, xcolor[border_color].pixel);
    XClearWindow(display, main_win);
    need_border_refresh=False;
}



/* SetBorderColor:
 *  Fixe la couleur du pourtour de l'cran.
 */
void SetBorderColor(int color)
{
    if (border_color != color)
    {
        border_color=color;
        need_border_refresh=True;
    }
}



/* DrawGPL:
 *  Affiche un Groupe Point Ligne (un octet de VRAM).
 */
static void DrawGPL(int addr, int pt, int col)
{
    register int i;
    unsigned int x,y,c1,c2;

    modified_cell[addr%CELL_COLS][addr/(CELL_COLS*8)]=True;

    c1=((col>>3)&7)+(((~col)&0x40)>>3);
    c2=(col&7)+(((~col)&0x80)>>4);

    x=(addr%CELL_COLS)*8;
    y=addr/CELL_COLS;

    for (i=0; i<8; i++)
        PutPixel(buffer, x+i, y, xcolor[pt&(0x80>>i) ? c1 : c2].pixel);
}


#define LED_SIZE 6*screen_size


/* SetDiskLed:
 *  Allume/teint la led du lecteur de disquettes.
 */
static void SetDiskLed(int led_on)
{
    if (led_on)
    {
	XGCValues values;

	values.foreground=xcolor[TO7_NCOLORS].pixel;
	XChangeGC(display, gc, GCForeground, &values);
	XDrawRectangle(display, screen_win, gc, WINDOW_WIDTH-LED_SIZE, 0, LED_SIZE-1, LED_SIZE-1);

	values.foreground=xcolor[TO7_NCOLORS+1].pixel;
	XChangeGC(display, gc, GCForeground, &values);
	XFillRectangle(display, screen_win, gc, WINDOW_WIDTH-LED_SIZE+1, 1, LED_SIZE-2, LED_SIZE-2);
    }
    else
    {
        /* RetraceScreen() laisse parfois la led allume... */
        need_border_refresh = 1;
    }
}



/* InitGraphic:
 *  Slectionne le visual, met en place la palette de couleurs et
 *  initialise le mcanisme de bufferisation (dirty rectangles).
 */
void InitGraphic(void)
{
    register int i,j;

    /* Recherche et slection du visual */ 
    if (!XMatchVisualInfo(display, screen, 16, TrueColor, &visualinfo))
	if (!XMatchVisualInfo(display, screen, 24, TrueColor, &visualinfo))
	    if (!XMatchVisualInfo(display, screen, 32, TrueColor, &visualinfo))
		if (!XMatchVisualInfo(display, screen, 8, PseudoColor, &visualinfo))
		{
		    fprintf(stderr,"%s: aucun visual disponible.\n",PROG_NAME);
		    exit(EXIT_FAILURE);
		}

    /* Initialisation de la palette de couleurs */
    colormap=DefaultColormap(display, screen);
    gc=DefaultGC(display, screen);

    /* rcupration de la palette du TO7-70 */
    for (i=0; i<TO7_NCOLORS; i++)
    {
        int red, green, blue;

        to7_QueryColor(i, &red, &green, &blue);

	xcolor[i].red   = 0x101*red;
	xcolor[i].green = 0x101*green;
	xcolor[i].blue  = 0x101*blue;
    }

    /* Couleurs de la led du lecteur de disquettes */
    xcolor[TO7_NCOLORS].red   = 0;
    xcolor[TO7_NCOLORS].green = 0;
    xcolor[TO7_NCOLORS].blue  = 0;

    xcolor[TO7_NCOLORS+1].red   = 0;
    xcolor[TO7_NCOLORS+1].green = 0xFFFF;
    xcolor[TO7_NCOLORS+1].blue  = 0;

    if (visualinfo.class==TrueColor)
        for (i=0;i<TO7_NCOLORS+2;i++)
	    XAllocColor(display, colormap, &xcolor[i]);
    else if (visualinfo.class == PseudoColor)
    {
	long int pixels[TO7_NCOLORS+2];

	if (!XAllocColorCells(display, colormap, False, 0, 0, pixels, TO7_NCOLORS+2))
	{
	    colormap=XCreateColormap(display, main_win, visualinfo.visual, AllocNone);

	    if (colormap == DefaultColormap(display, screen))
	    {
		fprintf(stderr,"%s: palette de couleurs immuable.\n",PROG_NAME);
		exit(EXIT_FAILURE);
	    }

	    XSetWindowColormap(display, main_win, colormap);
	}

	for (i=0; i<TO7_NCOLORS+2; i++)
	{
	    xcolor[i].pixel=pixels[i];
	    xcolor[i].flags=DoRed|DoGreen|DoBlue;
	    XStoreColor(display, colormap, &xcolor[i]);
	}
    }
	
    /* Cration du buffer d'affichage */
    if (mit_shm_enabled)
    {
	XShmSegmentInfo *shminfo = malloc(sizeof(XShmSegmentInfo));

        printf("Extension MIT-SHM utilise.\n");
        buffer=XShmCreateImage(display, visualinfo.visual, visualinfo.depth, ZPixmap, 0, shminfo, SCREEN_WIDTH, SCREEN_HEIGHT);

	shminfo->shmid=shmget(IPC_PRIVATE, SCREEN_HEIGHT*buffer->bytes_per_line, IPC_CREAT | 0777);
	shminfo->shmaddr= buffer->data= (char *) shmat(shminfo->shmid, 0, 0);
	shminfo->readOnly = False;

	XShmAttach (display, shminfo);
	XSync (display, 0);
	
	shmctl (shminfo->shmid, IPC_RMID, 0);
    }
    else
    {
	buffer=XCreateImage(display, visualinfo.visual, visualinfo.depth, ZPixmap, 0, NULL, SCREEN_WIDTH, SCREEN_HEIGHT, 32, 0);
	buffer->data=malloc(SCREEN_HEIGHT*buffer->bytes_per_line);
    }

#ifdef DEBUG
    fprintf(stderr, "buffer:  width=%d, height=%d\n", buffer->width, buffer->height);
    fprintf(stderr, "         format=%s\n", buffer->format==XYBitmap ? "XYBitmap" : (buffer->format==XYPixmap ? "XYPixmap" : "ZPixmap"));
    fprintf(stderr, "         byte_order=%s\n", buffer->byte_order==LSBFirst ? "LSBFirst" : "MSBFirst");
    fprintf(stderr, "         bitmap_unit=%d\n", buffer->bitmap_unit);
    fprintf(stderr, "         bitmap_bit_order=%s\n", buffer->bitmap_bit_order==LSBFirst ? "LSBFirst" : "MSBFirst");
    fprintf(stderr, "         bitmap_pad=%d\n", buffer->bitmap_pad);
    fprintf(stderr, "         depth=%d\n", buffer->depth);
    fprintf(stderr, "         bytes_per_lines=%d\n", buffer->bytes_per_line);
    fprintf(stderr, "         bits_per_pixel=%d\n",  buffer->bits_per_pixel);
    fprintf(stderr, "          red_mask=%lx\n",  buffer->red_mask);
    fprintf(stderr, "          green_mask=%lx\n",  buffer->green_mask);
    fprintf(stderr, "          blue_mask=%lx\n",  buffer->blue_mask);
#endif

    for (i=0; i<CELL_COLS+1; i++)
        for (j=0; j<CELL_ROWS; j++)
            modified_cell[i][j]=False;

    to7_SetBorderColor=SetBorderColor;
    to7_DrawGPL=DrawGPL;
    to7_SetDiskLed = SetDiskLed;

    if (screen_size==1)
        PutPixel=PutPixel1;
    else if (screen_size==2)
        PutPixel=PutPixel2;
}



/* RetraceScreen:
 *  Retrace une portion de l'cran.
 */
void RetraceScreen(int x, int y, int w, int h)
{
 
    if (mit_shm_enabled)
    {
        XShmPutImage(display, screen_win, gc, buffer, x, y, x, y, w, h, 1);
         while (!(XShmGetEventBase (display) + ShmCompletion));
    }
    else
        XPutImage(display, screen_win, gc, buffer, x, y, x, y, w, h);

  }



/* RefreshScreen:
 *  Rafrachit l'cran du TO7-70.
 */
void RefreshScreen(void)
{
    int i,j;
    int cell_start, cell_size=8*screen_size;

    for (j=0; j<CELL_ROWS; j++)
	for (i=0; i<CELL_COLS; i++)
	    if (modified_cell[i][j])
	    {
		cell_start=i;

		while (modified_cell[i][j])
		    modified_cell[i++][j]=False;

		RetraceScreen(cell_start*cell_size, j*cell_size, (i-cell_start)*cell_size, cell_size);
                
	    }
}
