/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999-2003 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : linux/sound.c
 *  Version    : 1.5.5
 *  Cr par   : Eric Botcazou aot 1999
 *  Modifi par: Eric Botcazou 03/11/2003
 * Modifi par : Thomas Missonier 05/07/2016
 * 
 * Modification 05/07/2016  : Remplacement de /dev/dsp par l'implmentation de Pulse Audio
 *
 *  Gestion de l'mulation sonore du TO7-70.
 */


#ifndef SCAN_DEPEND
   #include <stdio.h>
   #include <fcntl.h>
   #include <unistd.h>
   #include <sys/ioctl.h>
   #include <sys/soundcard.h>
#include <pulse/simple.h>
#include <pulse/error.h>
   #endif

#include "linux/main.h"
#include "to7.h"



//#define SOUND_FREQ  25600
#define SOUND_FREQ  25600
#define FRAG_EXP 9
#define SOUND_BUFFER_SIZE (1<<FRAG_EXP)
#define DEVNAME "/dev/dsp"

//Pulse
pa_simple *s;
 static const pa_sample_spec ss = {
        .format = PA_SAMPLE_U8,
        .rate = SOUND_FREQ,
        .channels = 1
    };
    

static unsigned char sound_buffer[SOUND_BUFFER_SIZE];
static int last_index;
static unsigned char last_data;

//Pulse
static int error;




/* PutSoundByte:
 *  Place un octet dans le tampon de streaming audio.
 */
static void PutSoundByte(unsigned long long int clock, unsigned char data)
{
    register int i;
    int index=(clock%TO7_CYCLES_PER_FRAME)*SOUND_FREQ/TO7_CPU_FREQ;

    /* Dans le cas o le nombre de cycles xcuts pendant une frame dpasse la valeur
       thorique, on bloque l'index  sa valeur maximale */
    if (index < last_index)
	index=SOUND_BUFFER_SIZE;

    for (i=last_index; i<index; i++)
        sound_buffer[i]=last_data;

    last_index=index;
    last_data=data;
}



/* InitSound:
 *  Initialise le module de streaming audio.
 */
void InitSound(void)
{

      to7_PutSoundByte=PutSoundByte;
     
     
    if (thom.sound_enabled)
    {
        printf("Initialisation du son...");
	fflush(stdout);
      //Pulse
        printf("\ninitialisation Pulse Audio...\n");
        if (!(s = pa_simple_new(NULL, "Thom", PA_STREAM_PLAYBACK, NULL, "Emulateur", &ss, NULL, NULL, &error))) {
        fprintf(stderr,": Echec initialisation Pulse Audio .. erreur:\n");
        thom.sound_enabled=0;
         return;  
        }
    }
    else printf ("Initialisation Pulse ok\n");
               
                 
    }




/* PlaySoundBuffer:
 *  Envoie le tampon de streaming audio  la carte son.
 */
void PlaySoundBuffer(void)
{
    register int i;

     /* on remplit la fin du buffer avec le dernier byte dpos */
    for (i=last_index; i<SOUND_BUFFER_SIZE; i++)
        sound_buffer[i]=last_data;

    last_index=0;
  if (thom.sound_enabled)
        if (pa_simple_write(s, sound_buffer, SOUND_BUFFER_SIZE, &error) < 0) {
          fprintf(stderr,": Echec lecture de son avec Pulseaudio:\n");
           } 
         
}
