/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999-2002 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : win/keybint.c
 *  Version    : 1.5.5
 *  Cr par   : Eric Botcazou octobre 1999
 *  Modifi par: Eric Botcazou 15/01/2002
 *
 *  Interface de gestion du clavier.
 */


#ifndef SCAN_DEPEND
   #include <allegro.h>
   #include <allegro/internal/aintern.h>
#endif

#include "alleg/main.h"
#include "to7.h"



/* KeyboardHandler:
 *  Handler des vnements bas-niveau clavier.
 */
static void KeyboardHandler(int key)
{
    int release=key&0x80;

    key&=0x7F;

    switch (key)
    {
        case KEY_ESC:
            if (!release)
                thom.command=CONTROL_PANEL;
            break;

        case KEY_F11:
            if (!release)
                thom.command=SCREENSHOT;
            break;

        default:
            to7_HandleKeyPress(key, release);
    }
}



/* InstallKeybint:
 *  Installe le module de gestion bas-niveau du clavier.
 */
void InstallKeybint(void)
{
    int mask=(1<<TO7_MAX_FLAG)-1, value=0;

    if (_key_shifts&KB_CAPSLOCK_FLAG)
        value |= TO7_CAPSLOCK_FLAG;

    if (_key_shifts&KB_SHIFT_FLAG)
        value |= TO7_SHIFT_FLAG;

    if (_key_shifts&KB_CTRL_FLAG)
        value |= TO7_CTRL_FLAG;

    to7_InputReset(mask, value);

    keyboard_lowlevel_callback=KeyboardHandler;
}



/* ShutDownKeybint:
 *  Dsinstalle le module de gestion bas-niveau du clavier.
 */
void ShutDownKeybint(void)
{
    keyboard_lowlevel_callback=NULL;
}



/* InitKeybint:
 *  Initialise le module de gestion bas-niveau clavier.
 */
void InitKeybint(void) {}

